
# HT-PAMDA fastqs2counts (script 1)
# generate raw PAM read counts from fastq files

#----------------------------------------------------------------------------------------------#

#input files and parameters:

import itertools
import gzip
import glob
import pandas as pd

#enter name of barcode CSV here
variant_ids = pd.read_csv('expRW086_pools_1-3_barcodes.csv') 
# barcode CSV should have the following headers, with example entries shown below:
# sample					fwd_barcode		rev_barcode		description
# unique_sample_id_01		CCTG			TGAC			WT_SpCas9
# unique_sample_id_02		ATTC			TACT			SpG
# unique_sample_id_03		CTAC			ACCG			SpRY

#fastq determines timepoint (except for t0 control)
#each fastq may contain multiple samles at a given timepoint
#timepoint fastq names are abbreviated as follows: timepoint1_S1_L001_R1_001.fastq.gz --> timepoint1_S1
timepoint_fastq = {'expRW086_pool_01_S1': 0, 'expRW086_pool_02_S2': 1, 'expRW086_pool_03_S3': 2}

pam_length = 4

fastqs = glob.glob('*R1*.fastq.gz') # grabs all fastqs in directory

spacer1 = 'EGFP1'
spacer2 = 'EGFP2'
spacers = {spacer1:'GGGCACGGGCAGCTTGCCGG', spacer2:'GTCGCCCTCGAACTTCACCT'} # spacer ids

#fwd and reverse barcodes, defined by the barcode sequences in the arrayed PAMDA primers used in PCR1
fwd_barcodes = ['CCTG',
'CGTT',
'ATTC',
'CTAC',
'TCTC',
'CCAT',
'GCAA',
'ATGC',
'GATG',
'CGAT']
rev_barcodes = ['TGAC',
'CATG',
'TACT',
'GCGA',
'AAGT',
'CCGT',
'TGGA',
'TTCG',
'ACAT',
'TCTA',
'CAAA',
'ACCG']


timepoints = [0, 60, 480, 1920] # timepoints used (seconds)

nt_complement = dict({'A':'T','C':'G','G':'C','T':'A','N':'N','_':'_','-':'-'})

def reverse_complement(seq):
	return "".join([nt_complement[c] for c in seq.upper()[-1::-1]])

nucleotides = ['A','T','C','G']
total_pam_space = [''.join(p) for p in itertools.product(nucleotides, repeat = pam_length)]

variant_dict = {}

for index, row in variant_ids.iterrows():
	variant_dict[ str(row['fwd_barcode']) + '_' + str(row['rev_barcode']) ] = row['sample']

#----------------------------------------------------------------------------------------------#

# Start counting PAMs

#iterate across fastqs, line by line, identify the timepoint, protein, spacer, and barcode, and record the count

print 'counting from fastqs', '\n'

store_all_data = {}
norm_counts_scale = {}

for sample in variant_ids['sample']:
	store_all_data[sample] = {spacer1:{x:[0]*(len(timepoints)-1) for x in total_pam_space}, spacer2: {x:[0]*(len(timepoints)-1) for x in total_pam_space}}

for fastq in fastqs:

	fastqR1 = fastq
	fastqR2 = fastq.replace('R1','R2')
	fastq_name = fastqR1.split('_L00')[0]

	print fastq_name, '\n'

	timepoint = timepoint_fastq[fastq_name]

	if fastqR1.endswith('.gz'):
		infileR1 = gzip.open(fastqR1, 'r')
		infileR2 = gzip.open(fastqR2, 'r')
	else:
		infileR1 = open(fastqR1, 'r')
		infileR2 = open(fastqR2, 'r')
	# QC variables:

	wrong_barcode = 0
	wrong_spacer = 0

	total_reads = 0
	counted_reads = 0
	level1 = 0
	level2 = 0
	level1rc = 0
	level2rc = 0
	
	while infileR1.readline() and infileR2.readline():
		read_sequenceR1 = infileR1.readline().strip()
		infileR1.readline()
		infileR1.readline()

		read_sequenceR2 = infileR2.readline().strip()
		infileR2.readline()
		infileR2.readline()

		total_reads += 1

		read_counted = False

		fwd_barcode = read_sequenceR1[2:6]
		rev_barcode = reverse_complement(read_sequenceR2[2:6])

		if fwd_barcode in fwd_barcodes and rev_barcode in rev_barcodes:
			barcode_pair = fwd_barcode + '_' + rev_barcode
			if barcode_pair in variant_dict.keys():
				spacer = reverse_complement(read_sequenceR1[39:59])
				if spacer == spacers[spacer1]:
					try:
						PAM = reverse_complement(read_sequenceR1[39 - pam_length:39])
						store_all_data[variant_dict[barcode_pair]][spacer1][PAM][timepoint] += 1
						counted_reads += 1
						read_counted = True
					except:
						continue

				elif spacer == spacers[spacer2]:
					try:
						PAM = reverse_complement(read_sequenceR1[39 - pam_length:39])
						store_all_data[variant_dict[barcode_pair]][spacer2][PAM][timepoint] += 1
						counted_reads += 1
						read_counted = True
					except:
						continue
				else:
					wrong_spacer +=1

		else:
			wrong_barcode +=1
		if total_reads % 500000 == 0:
			print round(float(counted_reads)/float(total_reads)*100,2) , '% of reads mapping'
			print total_reads, ' total reads'
			print round(100-float(wrong_barcode)/float(total_reads)*100,2), '% of barcodes correct'
			print round(100-float(wrong_spacer)/float(total_reads)*100,2), '% of spacers correct', '\n'

#----------------------------------------------------------------------------------------------#

#output raw count results as a csv

with open('pam_depletion_output_raw_counts_%sntPAM.csv'% pam_length, 'w') as f_out:
	f_out.write(','.join(map(str, ['Sample', 'Spacer', 'PAM'] + ['Raw_Counts_' + str(x) for x in range(1, len(timepoints))])) + '\n')
	for fastq in store_all_data:
		for spacer in store_all_data[fastq]:
			for pam in store_all_data[fastq][spacer]:
				total_info = [fastq, spacer, pam] + store_all_data[fastq][spacer][pam]
				f_out.write(','.join(map(str, total_info)) + '\n')

