
# HT-PAMDA rawcount2normcount (script 2)
# generate normalized PAM read counts from raw counts

#----------------------------------------------------------------------------------------------#

#input files and parameters:

import itertools
import pandas as pd
import numpy as np
from scipy.stats import linregress


control_sample = 'expRW060_160' #name of the untreated randomized PAM library control sample
#the control sample may be included in a fastq that corresponds to another timepiont
control_sample_timepoint_fastq = 3 

timepoints = [0, 60, 480, 1920] # timepoints used (s)

top_n = 5 # top n most enriched PAMs used to correct for uptrend

spacer1 = 'EGFP1'
spacer2 = 'EGFP2'
spacers = {spacer1:'GGGCACGGGCAGCTTGCCGG', spacer2:'GTCGCCCTCGAACTTCACCT'} # spacer ids

pam_length = 4

nt_complement = dict({'A':'T','C':'G','G':'C','T':'A','N':'N','_':'_','-':'-'})

def reverse_complement(seq):
    return "".join([nt_complement[c] for c in seq.upper()[-1::-1]])

nucleotides = ['A','T','C','G']
total_pam_space = [''.join(p) for p in itertools.product(nucleotides, repeat = pam_length)]


df = pd.read_csv('pam_depletion_output_raw_counts_%sntPAM.csv' % pam_length) #input file name

#----------------------------------------------------------------------------------------------#

#start normalizing read counts

print "normalizing counts"


#set up dataframes

for i in range(1, len(timepoints)):
	df['Norm_Counts_' + str(i)] = df['Raw_Counts_' + str(i)] / df.groupby(['Sample', 'Spacer'])['Raw_Counts_' + str(i)].transform(sum)

# determine t0 read counts for control sample

control_dict = {}
for spacer in spacers:
    control_dict[spacer] = {}
    for PAM in total_pam_space:
        control_dict[spacer][PAM]=0

for index, row in df.iterrows():
    if row['Sample'] == control_sample:
        control_dict[row['Spacer']][row['PAM']] = row['Norm_Counts_'+ str(control_sample_timepoint_fastq)]

norm_counts_0 = []
for index, row in df.iterrows():
    norm_counts_0.append(control_dict[row['Spacer']][row['PAM']])

df['Norm_Counts_0'] = norm_counts_0

#determing top n enriched PAMs per sample
uptrends = {}
x = range(len(timepoints))
for index, row in df.iterrows():
    sample_spacer = str(row['Sample']) + '_' + str(row['Spacer'])
    y = [row['Norm_Counts_' + str(i)] for i in range(len(timepoints))]
    slope = linregress(x, y)
    if sample_spacer in uptrends:
        uptrends[sample_spacer].append([slope[0], y])
    else:
        uptrends[sample_spacer] = [[slope[0], y]]

#determine correction for enrichment based on top n enriched PAMs per sample

uptrend_corrections = {}
for u in uptrends:
    uptrends[u] = sorted(uptrends[u])
    top_n_entries = [x[1] for x in uptrends[u][-top_n:]]
    top_n_entries_reformat = map(list, zip(*top_n_entries))
    top_n_entries_median = [np.median(x) for x in top_n_entries_reformat]
    uptrend_corrections[u] = top_n_entries_median

#calculate normalized read counts
#correct for enrichment
#normalize relative to t0 abundance

for index, row in df.iterrows():
    sample_spacer = str(row['Sample']) + '_' + str(row['Spacer'])
    for i in range(len(timepoints)):
        df.loc[index, 'Norm_Counts_' + str(i)] = row['Norm_Counts_' + str(i)] / uptrend_corrections[sample_spacer][i]
        df.loc[index, 'Norm_Counts_' + str(i)] = row['Norm_Counts_' + str(i)] / row['Norm_Counts_0']

df.to_csv('pam_depletion_output_norm_counts_%sntPAM.csv' % pam_length, index = False)