# HT-PAMDA rates2heatmap (script 4)
# generate 16x16 heatmap representation of PAM preference

#----------------------------------------------------------------------------------------------#

#input files and parameters:
import os
import itertools
import pandas as pd
import numpy as np
from scipy.stats import linregress
import seaborn as sns
import matplotlib.pyplot as plt
plt.switch_backend('agg')

pam_length = 4

variant_ids = pd.read_csv('expRW086_pools_1-3_barcodes.csv') #sample barcode file input

#average across replicates or different spacer sequences
replicates = []
for variant in variant_ids['sample']:
    replicates.append([variant])

variant_name_dict = {}
for index, row in variant_ids.iterrows():
    variant_name_dict[row['sample']] = row ['description']

if not os.path.exists('HT_PAMDA_heatmaps'):
    os.mkdir('HT_PAMDA_heatmaps')
 

use_timepoints = [[0,1,2,3]] 

#----------------------------------------------------------------------------------------------#

for timepoints in use_timepoints:
 
    csv_input = 'pam_depletion_output_rates_%sntPAM_%s.csv' % (pam_length, timepoints)

    #determine heatmap design, default is positions 1 and 2 on y-axis and 3 and 4 on x-axis
    #these parameters determine the dimensions and nucleotide ordering/priority of the heatmap
    split_pam_index = 2 #counts from left to right
    pam1_nucleotide_rank = {1:'A',2:'C',3:'G',4:'T'}  #don't change this without changing below as well
    pam2_nucleotide_rank = {1:'A',2:'C',3:'G',4:'T'}  #don't change this without changing below as well
    pam1_index_rank = [2,1] # Rows                    #don't change this without changing below as well
    pam2_index_rank = [1,2] # Columns                 #don't change this without changing below as well
    #make sure dimensions of these lists match the length of the PAM on either split of the PAM

    # Reformatting input dataframe
    df_input = pd.read_csv(csv_input)
    df_input['PAM_pt1'] = [x[:split_pam_index] for x in df_input['PAM'].tolist()]
    df_input['PAM_pt2'] = [x[split_pam_index:] for x in df_input['PAM'].tolist()]

    spacers = df_input['Spacer'].unique().tolist()

    pam_length = len(df_input['PAM'].tolist()[0])

    numbers = ['1','2','3','4']
    pam_space = [''.join(p) for p in itertools.product(numbers, repeat = pam_length)]

    # Sort PAMs according to rules
    pam1_ids = []
    pam2_ids = []
    for pam in pam_space:
        pam1_ids.append(int(pam[:split_pam_index]))
        pam2_ids.append(int(pam[split_pam_index:]))

    pam1_ids = sorted([int(x) for x in set(pam1_ids)])
    pam2_ids = sorted([int(x) for x in set(pam2_ids)])
        
    columns = []
    indices = []
    tmp = ''
    for pam in pam1_ids:
        tmp_dict = {i:j for i,j in zip(pam1_index_rank, list(str(pam)))}
        for i in sorted(tmp_dict):
            tmp += pam1_nucleotide_rank[int(tmp_dict[i])]
        indices.append(tmp)
        tmp = ''

    tmp = ''
    for pam in pam2_ids:
        tmp_dict = {i:j for i,j in zip(pam2_index_rank, list(str(pam)))}
        for i in sorted(tmp_dict):
            tmp += pam2_nucleotide_rank[int(tmp_dict[i])]
        columns.append(tmp)
        tmp = ''

    # Construct dataframes for different replicates

    new_columns = []
    for replicate_group in sorted(replicates):
        print replicate_group[0]," - ", variant_name_dict[replicate_group[0]]
        print " _______  _______  __   __  ______   _______ "
        print "|       ||   _   ||  |_|  ||      | |   _   |"
        print "|    _  ||  |_|  ||       ||  _    ||  |_|  |"
        print "|   |_| ||       ||       || | |   ||       |"
        print "|    ___||       ||       || |_|   ||       |"
        print "|   |    |   _   || ||_|| ||       ||   _   |"
        print "|___|    |__| |__||_|   |_||______| |__| |__|"

        for column in columns:
        
            for replicate in replicate_group:
            
                new_columns.append(str(column) + '-' + str(replicate))

        df_output = pd.DataFrame(columns = new_columns, index = indices)
                
        for row in df_output.index:

            for column in df_output.columns:
                
                pam2 = str(column.split('-')[0].strip('\n'))
                sample = str(column.split('-')[1].strip('\n'))

                rate_avg = 0
                for spacer in sorted(spacers):
                    rate_avg += float(df_input['Rate_Constant_k'][(df_input['PAM_pt1']==str(row))&(df_input['PAM_pt2']==pam2)&(df_input['Sample']==sample)&(df_input['Spacer']==spacer)].tolist()[0])
                rate_avg = rate_avg/len(spacers)
                
                df_output.loc[row, column] = np.log10(rate_avg)

#determining heatmap scale
#change these parameters to alter heatmap scale

        rate_list = []              
        bottom_n_rates_for_min = 32
        top_n_rates_for_max = 8
        for row in df_output.index:
            for column in df_output.columns:
                rate_list.append(df_output.loc[row,column])
        rate_list = sorted(rate_list)

        heatmap_min = sum(rate_list[:bottom_n_rates_for_min-1])/bottom_n_rates_for_min*0.65
        heatmap_max = sum(rate_list[len(rate_list)-top_n_rates_for_max:])/top_n_rates_for_max
        
        df_output = df_output[df_output.columns].astype(float)

#generate heatmaps as PDFs, CSVs
        sns.set(font_scale = 1)

        fig, ax = plt.subplots()
        plt.title(replicate_group[0] + '(' + variant_name_dict[replicate_group[0]] + ')' , y=1.2)
        ax = sns.heatmap(df_output, vmin = heatmap_min, vmax = heatmap_max, square = True, cmap ='Blues', cbar = True, cbar_kws={"shrink": 1}, linewidth = 0.2, linecolor = "White",  xticklabels = ['AA','AC','AG','AT','CA','CC','CG','CT','GA','GC','GG','GT','TA','TC','TG','TT'])
        ax.xaxis.tick_top()
        plt.yticks(rotation=0)
        plt.xticks(rotation=90)

        #save heatmap files. save as PDP , png , or csv
        plt.savefig('HT_PAMDA_heatmaps/HEATMAP_%sntPAM_split@nt%s_%s_%s_%s.pdf' % (pam_length, split_pam_index, replicate_group[0], timepoints, ('(' + variant_name_dict[replicate_group[0]] + ')') ))
        plt.savefig('HT_PAMDA_heatmaps/HEATMAP_%sntPAM_split@nt%s_%s_%s_%s.png' % (pam_length, split_pam_index, replicate_group[0], timepoints, ('(' + variant_name_dict[replicate_group[0]] + ')') ), dpi = 400)
        df_output.to_csv('HT_PAMDA_heatmaps/HEATMAP_%sntPAM_split@nt%s_%s_%s.csv' % (pam_length, split_pam_index, '_'.join(map(str, replicate_group)), timepoints ))
        new_columns = []
